function [x,p] = solve_iStokes_Mini(coordinates,elements, ...
                                    dirichlet,neumann,uD,f,g,nu)
%
% Mini Element (P1 + Bubble)^2 x P1
%
%*** initiate a few constants
nT = size(elements,1);
nC = size(coordinates,1);
%*** First vertex of elements and corresponding edge vectors 
c1  = coordinates(elements(:,1),:);
d21 = coordinates(elements(:,2),:) - c1;
d31 = coordinates(elements(:,3),:) - c1;
%*** Vector of element areas |T|
area = abs((d21(:,1).*d31(:,2)-d21(:,2).*d31(:,1))/2);
%*** Assembly of stiffness matrix
telem = [elements,nC+(1:nT)'];
I = reshape(telem(:,[1 2 3 1 2 3 1 2 3 4]),[],1);
J = reshape(telem(:,[1 1 1 2 2 2 3 3 3 4]),[],1);
a = 1/4*(sum(d31.*d31,2)./area);
b = 1/4*(sum(d21.*d31,2)./area);
c = 1/4*(sum(d21.*d21,2)./area);
A = nu*sparse(I,J,[a-2*b+c; b-a; b-c; b-a; a; -b; b-c; -b; c; a-b+c]);
%*** Assembly of divergence matrix
I = reshape(telem(:,[1 2 3 4 1 2 3 4 1 2 3 4]),[],1);
J = reshape(telem(:,[1 1 1 1 2 2 2 2 3 3 3 3]),[],1);
a = d31/6; b = d31/(4*sqrt(10)); c = d21/6; d=d21/(4*sqrt(10)); 
B = [c-a;a;-c;b-d;c-a;a;-c;-b;c-a;a;-c;d];    
B = sparse([I;nC+nT+I],[J,J], [B(:,2),-B(:,1)]);  
%*** global matrix
A = [[A,sparse(nC+nT,nC+nT);sparse(nC+nT,nC+nT),A],-B;-B',sparse(nC,nC)];
%*** Prescribe values at Dirichlet nodes
dirichlet = unique(dirichlet);
x = zeros(2*(nC+nT)+nC,1);
x([dirichlet;nC+nT+dirichlet]) = reshape(uD(coordinates(dirichlet,:)),[],1);
%*** Computation of free and fixed nodes
freenodes = setdiff(1:(nC+nT), dirichlet);
freenodes = [freenodes,nC+nT+freenodes,2*(nT+nC)+(1:(nC+isempty(neumann)))];
%*** Assembly of right-hand side
b = zeros(2*(nC+nT)+nC,1);
if ~isempty(f)
  %*** Assembly of mass matrix (P1 + Bubble) x P1
  I = reshape(telem(:,[1 2 3 4 1 2 3 4 1 2 3 4]),[],1);
  J = reshape(telem(:,[1 1 1 1 2 2 2 2 3 3 3 3]),[],1);
  a = 1/12; c = 1/(6*sqrt(10));
  M = sparse(I,J,area*[2*a a a c a 2*a a c a a 2*a c]);
  fsT = feval(f,coordinates);
  b(1:2*(nC+nT),1) = reshape(M*fsT,[],1); 
end
b = b - A*x;
if ~isempty(neumann) 
  cn1 = coordinates(neumann(:,1),:);
  cn2 = coordinates(neumann(:,2),:);
  gmE = feval(g,(cn1+cn2)/2);
  tmp = repmat((sqrt(sum((cn2-cn1).^2,2))*[1/2,1/2]).*gmE,2,1);
  b = b + accumarray([neumann(:);nC+nT+neumann(:)],tmp(:),[size(b,1) 1]);
else
  C = sparse(elements(:),ones(3*nT,1),area(:,[1,1,1])/3);
  A(2*(nC+nT)+(1:nC),end+1) = C;
  A(end+1,2*(nC+nT)+(1:nC)) = C';
  b = [b;0]; x = [x;0];
end
%*** Computation of FEM approximation
x(freenodes) = A(freenodes,freenodes)\b(freenodes);
p = x(2*(nC+nT)+(1:nC));
x = reshape(x(1:2*(nC+nT)),[],2);

